#include <iostream>
#include <cstdlib>

#include "enums.hpp"

using namespace std;


struct ColorMapEntry
{
  Led c1;
  LED c2;
};


template<typename Color> // ELedColour
inline Color mapGetValue(const ColorMapEntry e)
{
  return e.c1;
}
template<>
inline LED mapGetValue<LED>(const ColorMapEntry e)
{
  return e.c2;
}

template<typename T, unsigned int N>
unsigned int sizeofTab(const T (&)[N])
{
  return N;
}

// main implementation call
template<typename To, typename From>
To mapColorEnum(const From from)
{
  const int           defaultIndex=0;
  // bidirectional map of colors (HAL<->VirtualDevice)
  const ColorMapEntry map[]={
                              { Led_None,  LED_NONE  },
                              { Led_Red,   LED_RED   },
                              { Led_Green, LED_GREEN },
                              { Led_Blue,  LED_BLUE  }
                            };

  // check if color is defined
  for(unsigned int i=0; i<sizeofTab(map); ++i)
    if( mapGetValue<From>(map[i])==from )
      return mapGetValue<To>(map[i]);

  // default entry, if no excalt match
  return mapGetValue<To>(map[defaultIndex]);
}


int main(int argc, char **argv)
{
  if(argc!=1+1)
  {
    cerr<<argv[0]<<" <id>"<<endl;
    return 1;
  }

  const int id  =atoi(argv[1]);
  const LED in  =(LED)id;
  const Led out1=mapColorEnum<Led, LED>(in);
  const LED out2=mapColorEnum<LED, Led>(out1);

  cout<<in<<" is "<<out1<<" and back to "<<out2<<endl;
  return 0;
}
